"use strict";
/**
 * @fileOverview the class of Helix Coordinate
 * @author sima.zhang
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var matrix_util_1 = require("@antv/matrix-util");
var util_1 = require("@antv/util");
var base_1 = require("./base");
var Helix = /** @class */ (function (_super) {
    tslib_1.__extends(Helix, _super);
    function Helix(cfg) {
        var _this = _super.call(this, tslib_1.__assign({ startAngle: 1.25 * Math.PI, endAngle: 7.25 * Math.PI, innerRadius: 0 }, cfg)) || this;
        _this.type = 'helix';
        _this.isHelix = true;
        _this._init();
        return _this;
    }
    Helix.prototype._init = function () {
        var width = this.width;
        var height = this.height;
        var radius = this.radius;
        var innerRadius = this.innerRadius;
        var startAngle = this.startAngle;
        var endAngle = this.endAngle;
        var index = (endAngle - startAngle) / (2 * Math.PI) + 1; // 螺线圈数
        var maxRadius = Math.min(width, height) / 2;
        if (radius && radius >= 0 && radius <= 1) {
            maxRadius = maxRadius * radius;
        }
        var d = Math.floor((maxRadius * (1 - innerRadius)) / index);
        var a = d / (Math.PI * 2); // 螺线系数
        var x = {
            start: startAngle,
            end: endAngle,
        };
        var y = {
            end: innerRadius * maxRadius + d * 0.99,
            start: innerRadius * maxRadius,
        };
        this.a = a;
        this.d = d;
        this.x = x;
        this.y = y;
    };
    Helix.prototype.getCenter = function () {
        return this.center;
    };
    /**
     * 将百分比数据变成屏幕坐标
     * @param  {Object} PointType 归一化的点坐标
     * @return {Object}       返回对应的屏幕坐标
     */
    Helix.prototype.convertPoint = function (point) {
        var a = this.a;
        var center = this.center;
        var x;
        var y;
        if (this.isTransposed) {
            x = point.y;
            y = point.x;
        }
        else {
            x = point.x;
            y = point.y;
        }
        var thi = this.convertDim(x, 'x');
        var r = a * thi;
        var newY = this.convertDim(y, 'y');
        return {
            x: center.x + Math.cos(thi) * (r + newY),
            y: center.y + Math.sin(thi) * (r + newY),
        };
    };
    /**
     * 将屏幕坐标点还原成百分比数据
     * @param  {Object} PointType 屏幕坐标
     * @return {Object}       返回对应的归一化后的数据
     */
    Helix.prototype.invertPoint = function (point) {
        var center = this.center;
        var a = this.a;
        var d = this.d + this.y.start;
        var v = matrix_util_1.vec2.subtract([], [point.x, point.y], [center.x, center.y]);
        var thi = matrix_util_1.vec2.angleTo(v, [1, 0], true);
        var rMin = thi * a; // 坐标与原点的连线在第一圈上的交点，最小r值
        if (matrix_util_1.vec2.length(v) < rMin) {
            // 坐标与原点的连线不可能小于最小r值，但不排除因小数计算产生的略小于rMin的情况
            rMin = matrix_util_1.vec2.length(v);
        }
        var index = Math.floor((matrix_util_1.vec2.length(v) - rMin) / d); // 当前点位于第index圈
        thi = 2 * index * Math.PI + thi;
        var r = a * thi;
        var newY = matrix_util_1.vec2.length(v) - r;
        newY = util_1.isNumberEqual(newY, 0) ? 0 : newY;
        var x = this.invertDim(thi, 'x');
        var y = this.invertDim(newY, 'y');
        x = util_1.isNumberEqual(x, 0) ? 0 : x;
        y = util_1.isNumberEqual(y, 0) ? 0 : y;
        var rst = { x: 0, y: 0 };
        rst.x = this.isTransposed ? y : x;
        rst.y = this.isTransposed ? x : y;
        return rst;
    };
    return Helix;
}(base_1.default));
exports.default = Helix;
//# sourceMappingURL=helix.js.map